"""Create alerts models

Revision ID: 0d7bea8f4d60
Revises: fc391d505594
Create Date: 2025-08-04 22:33:08.875952

"""

from typing import Sequence, Union

import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "0d7bea8f4d60"
down_revision: Union[str, None] = "fc391d505594"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "alert_definition",
        sa.Column("properties", postgresql.JSON(astext_type=sa.Text()), nullable=True),
        sa.Column("type", sa.SmallInteger(), nullable=True),
        sa.Column("level", sa.SmallInteger(), nullable=True),
        sa.Column("device_name", sa.Text(), nullable=True),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("tenant", sa.UUID(), nullable=True),
        sa.Column("external_id", sa.Text(), nullable=True),
        sa.Column("meter_id", sa.Text(), nullable=True),
        sa.Column("created_ts", sa.DateTime(), nullable=True),
        sa.Column("updated_ts", sa.DateTime(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("tenant", "meter_id"),
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("alert_definition")
    # ### end Alembic commands ###



"""Add alert model

Revision ID: 1646629c1c90
Revises: 314a5d69646a
Create Date: 2025-08-13 21:08:28.462217

"""

from typing import Sequence, Union

import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "1646629c1c90"
down_revision: Union[str, None] = "314a5d69646a"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "alert",
        sa.Column("detected_ts", sa.DateTime(), nullable=False),
        sa.Column("active_marker", sa.Boolean(), nullable=True),
        sa.Column("resolved_ts", sa.DateTime(), nullable=True),
        sa.Column("details", postgresql.JSON(astext_type=sa.Text()), nullable=True),
        sa.Column("alert_definition_id", sa.Integer(), nullable=True),
        sa.Column("type", sa.SmallInteger(), nullable=True),
        sa.Column("level", sa.SmallInteger(), nullable=True),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("tenant", sa.UUID(), nullable=True),
        sa.Column("external_id", sa.Text(), nullable=True),
        sa.Column("meter_id", sa.Text(), nullable=True),
        sa.Column("created_ts", sa.DateTime(), nullable=True),
        sa.Column("updated_ts", sa.DateTime(), nullable=True),
        sa.ForeignKeyConstraint(
            ["alert_definition_id"],
            ["alert_definition.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint(
            "tenant",
            "meter_id",
            "type",
            "level",
            "active_marker",
            name="unique_active_alert_per_device",
        ),
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("alert")
    # ### end Alembic commands ###



"""Delete alert uniqueness

Revision ID: 90c1158cc2ba
Revises: 0d7bea8f4d60
Create Date: 2025-08-05 00:17:10.987834

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "90c1158cc2ba"
down_revision: Union[str, None] = "0d7bea8f4d60"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(
        "alert_definition_tenant_meter_id_key", "alert_definition", type_="unique"
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_unique_constraint(
        "alert_definition_tenant_meter_id_key",
        "alert_definition",
        ["tenant", "meter_id"],
    )
    # ### end Alembic commands ###



from sqlalchemy import Boolean, Column, SmallInteger
from sqlalchemy.dialects.postgresql import JSON

from .base import Base, IdentityBase, Sources, TimeSign, Types


class RawMeasurement(IdentityBase, TimeSign, Base):
    __tablename__ = "measurement_raw"

    data = Column(JSON)
    type = Column(SmallInteger)
    source = Column(
        SmallInteger, nullable=False, server_default=str(Sources.sensorhub.value)
    )
    is_processed = Column(Boolean, default=False)

    @property
    def source_enum(self):
        return Sources(self.source)

    @source_enum.setter
    def source_enum(self, source_enum):
        self.source = source_enum.value

    @property
    def type_enum(self):
        return Types(self.type)

    @type_enum.setter
    def type_enum(self, type_enum):
        self.type = type_enum.value



"""Add source field to RawMeasurement.

Revision ID: e0d5bb6c3974
Revises: 1646629c1c90
Create Date: 2026-01-06 21:30:21.741949

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op
from iot_db.models.base import Sources

# revision identifiers, used by Alembic.
revision: str = "e0d5bb6c3974"
down_revision: Union[str, None] = "1646629c1c90"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    # Add column as nullable first
    op.add_column(
        "measurement_raw", sa.Column("source", sa.SmallInteger(), nullable=True)
    )
    # Set default value (sensorhub) for existing rows
    op.execute(
        f"UPDATE measurement_raw SET source = {Sources.sensorhub.value} WHERE source IS NULL"
    )
    # Make column NOT NULL with server default
    op.alter_column(
        "measurement_raw",
        "source",
        nullable=False,
        server_default=str(Sources.sensorhub.value),
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    # Remove server default before dropping column
    op.alter_column("measurement_raw", "source", server_default=None)
    op.drop_column("measurement_raw", "source")
    # ### end Alembic commands ###
